<?php

function get_file_icon($file){
    $ext=strtolower(pathinfo($file, PATHINFO_EXTENSION));
    $icon="file-base.svg";
    switch($ext){
        case "jpg":
        case "webp":
        case "png":
        case "gif":
            $im=new thumbnail_tool(CHEMIN,CACHE_VIGNETTES,APP."images/");
            $im->set_source($file);
            $icon= $im->vignette(300,300,false);
            break;
        case "svg":
            $icon=getcompletefilepath($file);
            break;
        case "mp3":
        case "aac":
            $icon="app/images/file-music.svg";
            break;
        case "mp4":
        case "avi":
            $icon="app/images/file-movie.svg";
            break;
        case "zip":
            $icon="app/images/file-zip.svg";
            break;
        case "pdf":
            $icon="app/images/file-pdf.svg";
            break;
        default:
            $icon="app/images/file-earmark.svg";

    }
    return $icon;
} 

/*  
Classe de manipulation d'image
reçoit un dossier par défaut des images 
setsource envoie une adresse 

usage
$im=new thumbnail_tool(CHEMIN,CACHE_VIGNETTES,APP."images/");
$im->set_source($file);
$im->vignette(VIGNETTE_W,VIGNETTE_H,true)
$im->vignette(VIGNETTE_W,VIGNETTE_H) // le crop est true par défaut 

*/
class thumbnail_tool {
    public $im_vd="defaut.png";
    public $im_vdo="defaut-oversize.png";

    public $src_completepath; // depuis la racine du serveur
    public $src_publicpath; // depuis la racine du site
    public $src_w;
    public $src_h;

    public $root;

    public $src_publicfolder=false;
    public $src_serverfolder=false;

    public $vignette_publicfolder=false; // adresse serveur de la vignette
    public $vignette_serverfolder=false; // adresse public de la vignette

    public $maxsize=10000000000; // taille maximale de l'image en octet
    public $vignette_w=10000; // largeur max en pixels
    public $vignette_h=100000; // hauteur max en pixels
    public $vignette_crop=false;

    public $im_info;
    public $extension;
    public $size;

    /*
    constructeur: 
    fournir le dossier source $imsf 
    le dossier des vignettes  $imvf
    et le dossier des images par défaut $im_df
    */
    function __construct($server=__DIR__."/", $im_vi_folder="",$def_folder="" ) {

        if(!is_dir($def_folder) ){
            return false;
        }
        $def_folder=str_replace($server,"",$def_folder);

        $this->im_vd=$def_folder.$this->im_vd;
        $this->im_vdo=$def_folder.$this->im_vdo;

        // racine du serveur
        $this->root=$server;

        // créer les serverpath et publicpath
        // on assume que le sf et vf sont fournis en public link
        $vf_public=str_replace($server,"",$im_vi_folder);
        $this->vignette_publicfolder=$vf_public;
        $this->vignette_serverfolder=$this->root.$vf_public;
    }

    public function set_source($imbase){

        // nettoyer le lien serveur pour obtenir le lien public
        $imbase=str_replace($this->root,"",$imbase);
        // si l'image n'existe par, préparer la vignette par défaut
        if($imbase=="" || !is_file($this->root.$imbase) ){
            $imbase=$this->im_vd;
        }

        $this->src_publicpath=$imbase;
        $this->src_completepath=$this->root.$imbase;


        $this->im_info=pathinfo($this->src_completepath);
        $this->extension=$this->im_info['extension'];

        $this->src_publicfolder=$this->im_info['dirname']."/";
        $this->src_serverfolder=$this->root.$this->im_info['dirname']."/";

        // donner déjà une vignette par défaut
        $this->size=filesize($this->src_completepath);
        if($this->size > $this->maxsize){
            $vignette_spath=$this->im_vdo;
        } else {
            $vignette_spath=$this->im_vd;
        }

        list($width, $height, $type, $attr) = getimagesize($this->src_completepath);

        $this->src_w=$width;
        $this->src_h=$height;
    }

    public function vignette($dw=false,$dh=false,$crop=true){
        if(!$dw){ $dw= $this->vignette_w; }
        if(!$dh){ $dw= $this->vignette_h; }
        return $this->get_vignette($dw,$dh,$crop);
    }

    private function get_vignette($dw,$dh,$crop){
        $ext_image=array('jpg','png','gif','webp');
        if($this->extension=="svg" || !in_array($this->extension,$ext_image)){ 
            return $this->im_vd; 
        }
        // si l'image est plus petite que la vignette, renvoyer l'original
        if($this->src_w < $dw || $this->src_h < $dh){ 
            return $this->src_publicpath."?".filectime($this->src_completepath);
        }

        // c'est parti !
        $suffixe="_".$dw."x".$dh.($crop ? "_crop" : "_nocrop")."_".hash('adler32',filectime($this->src_completepath));
        $vignettefile=$this->im_info['basename'].$suffixe.".".$this->extension;

        // voir si la vignette existe déjà, si oui la renvoyer directement
        if(file_exists($this->vignette_serverfolder.$vignettefile)){
            return $this->vignette_publicfolder.$vignettefile."?".filectime($this->vignette_serverfolder.$vignettefile);
        } 

        switch($this->extension){
            case 'gif': $img = imagecreatefromgif($this->src_completepath); break;
            case 'jpg': $img = imagecreatefromjpeg($this->src_completepath); break;
            case 'png': $img = @imagecreatefrompng($this->src_completepath); break;
            case 'webp': $img = imagecreatefromwebp($this->src_completepath); break;
            default : echo "image does not exists"; return $image_default;
        }
        if($crop){
            $ratio = max($dw/$this->src_w, $dh/$this->src_h);
            $y= ($this->src_h - $dh / $ratio) /2;
            $x = ($this->src_w - $dw / $ratio) / 2;
            $nheight = $dh / $ratio;
            $nwidth = $dw / $ratio;
        } else {
            $ratio = min($dw/$this->src_w, $dh/$this->src_h);
            $dw = $this->src_w * $ratio;
            $dh = $this->src_h * $ratio;
            $x = 0;
            $y = 0;
            $nheight = $this->src_h;
            $nwidth = $this->src_w;
        }

        $vignette = imagecreatetruecolor($dw, $dh);

        // preserve transparency
        if($this->extension == "gif" || $this->extension == "png"){
            imagecolortransparent($vignette, imagecolorallocatealpha($vignette, 0, 0, 0, 127));
            imagealphablending($vignette, false);
            imagesavealpha($vignette, true);
        }

        imagecopyresampled($vignette, $img, 0, 0, $x, $y, $dw, $dh, $nwidth, $nheight);
        //$sharpen = array([-1,-1,-1], [-1, 16, -1], [-1,-1,-1]);
        //imageconvolution($vignette, $sharpen, 8, 0);

        switch($this->extension){
            case 'gif': imagegif($vignette, $this->vignette_serverfolder.$vignettefile); break;
            case 'jpg': imagejpeg($vignette, $this->vignette_serverfolder.$vignettefile); break;
            case 'png': imagepng($vignette, $this->vignette_serverfolder.$vignettefile); break;
            case 'webp': imagewebp($vignette, $this->vignette_serverfolder.$vignettefile); break;
        }
        imagedestroy($vignette);
        imagedestroy($img);

        return $this->vignette_publicfolder.$vignettefile."?".filectime($this->vignette_serverfolder.$vignettefile);

    }

    public function human_size($decimals = 2) {
        $factor = floor((strlen($this->size) - 1) / 3);
        if ($factor > 0) $sz = 'KMGT';
        return sprintf("%.{$decimals}f", $this->size / pow(1024, $factor)) . @$sz[$factor - 1] . 'B';
    }

}